
class FunctionNode:
    
    fun = None
    children = {}

all_functions = {}

# -----------------------------------------------------------------------------

'''
Created on 3. feb. 2015

@author: http://code.activestate.com/recipes/465831/
@author: 3 PORT: added "clean times", changed get_profile_stats
'''

from time import time, clock
import threading
import sys
from collections import deque

try:
    from resource import getrusage, RUSAGE_SELF
except ImportError:
    RUSAGE_SELF = 0
    def getrusage(who=0):
        return [0.0, 0.0] # on non-UNIX platforms cpu_time always 0.0

p_stats = None
p_start_time = None

def profiler(frame, event, arg):
    if event not in ('call','return'): return profiler
    #### gather stats ####
    rusage = getrusage(RUSAGE_SELF)
    t_cpu = rusage[0] + rusage[1] # user time + system time
    code = frame.f_code 
    fun = (code.co_name, code.co_filename, code.co_firstlineno)
    #### get stack with functions entry stats ####
    ct = threading.currentThread()
    try:
        p_stack = ct.p_stack
    except AttributeError:
        ct.p_stack = deque()
        p_stack = ct.p_stack
    try:
        time_stack = ct.time_stack
    except AttributeError:
        ct.time_stack = deque()
        time_stack = ct.time_stack
    #### handle call and return ####
    if event == 'call':
        #print "call " + code.co_name
        p_stack.append((time(), t_cpu, fun))
        time_stack.append(0)
    elif event == 'return':
        #print "return " + code.co_name
        try:
            t,t_cpu_prev,f = p_stack.pop()
            assert f == fun
        except IndexError: # TODO investigate
            t,t_cpu_prev,f = p_start_time, 0.0, None
        call_cnt, t_sum, t_cpu_sum, dt_clean_sum = p_stats.get(fun, (0, 0.0, 0.0, 0.0))
        dt = time() - t
        
        #
        dt_clean = 0.0;
        if len(time_stack)>0:
            dt_clean = dt + time_stack[-1] # subtract descendants' time
            if len(time_stack)>1:
                time_stack[-2] -= dt
            time_stack.pop()

        
        p_stats[fun] = (call_cnt+1, t_sum + dt, t_cpu_sum+t_cpu-t_cpu_prev, dt_clean_sum + dt_clean)
                
    return profiler


def profile_on():
    global p_stats, p_start_time
    p_stats = {}
    p_start_time = time()
    threading.setprofile(profiler)
    sys.setprofile(profiler)


def profile_off():
    threading.setprofile(None)
    sys.setprofile(None)

def get_profile_stats():
    """
    returns a sorted list of strings containing call statistics
    where
      each item represents a comma delimited line containing:
        real_time_clean, real_time, cpu_time, call_count, func_name, file_name, line_num
    
    All times are real numbers in seconds:
    
    real_time_clean = measured time inside a method excluding real_time made in calls to sub-methods
    real_time = measured time inside a method including real_time made in calls to sub-methods
    cpu_time = the CPU time of the method (does not work in Windows)
    call_count = the number of method invocations during profiling
    func_name = the name of the profiled method
    file_name = the name of the file containing the profiled method
    line_num = the line number of the profiled method 
    """
    lines = []
    for key, value in p_stats.iteritems():
        func_name = key[0]
        file_name = key[1]
        line_num = key[2]
        call_count = value[0]
        real_time = value[1]
        cpu_time = value[2] # works only on UNIX
        real_time_clean = value[3]
        
        lines.append([real_time_clean, real_time, cpu_time, call_count, func_name, file_name, line_num])
    
    #import platform
    #sort_by = 1 # sort by CPU time on UNIX only
    #if platform.system()=="Windows":
    #    sort_by = 0
    sort_by = 0
    
    # sort
    lines = sorted(lines, key=lambda x: x[sort_by])
    
    strings = []
    for line in lines:
        strings.append(", ".join([str(x) for x in line]))
    
    return strings

# -----------------------------------------------------------------------------

'''
Created on 6. feb. 2015

@author: 3 PORT - a micro benchmark utility
'''

class TimingPool:
    
    __timings = {}
    __invocation_counts = {}
    
    def add_timing(self, tag, dt):
        """tag is a string, dt is a float number representing seconds"""
        self.__timings[tag] = self.__timings.get(tag, 0.0) + dt
        self.__invocation_counts[tag] = self.__invocation_counts.get(tag, 0) + 1
        
    def get_stats(self):
        lines = []
        for key, value in self.__timings.iteritems():
            lines.append([key, value, self.__invocation_counts[key]])
        
        lines = sorted(lines, key = lambda x: x[1])
        strings = []
        for line in lines:
            strings.append(": ".join([line[0], "{0:10.6f}".format(line[1]), str(line[2])]))
    
        return "\n".join(strings)

# -----------------------------------------------------------------------------    

timing_poool = TimingPool()        

# -----------------------------------------------------------------------------

class SimpleTimer:

    __tag = None    
    __t1 = 0;
    
    def __init__(self, tag="undefined"):
        self.__tag = tag
        self.start()
    
    def start(self):
        self.__t1 = clock()
        
    def get_elapsed(self):
        """returns the elapsed time from start to now in seconds."""
        return clock() - self.__t1
    
    def publish(self):
        timing_poool.add_timing(self.__tag, self.get_elapsed());

