from django.shortcuts import render_to_response
from django.template.loader import render_to_string
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib import auth
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.template import RequestContext

from project.main.models import CustomUser, UserPermissions
from main.models import UserSubscriptions


#--------------------------------------------------------------------------------------------------

def _username_exists(username):
    return CustomUser.objects.filter(username = username).count() != 0

#--------------------------------------------------------------------------------------------------

def _valid_email(email):

    import re
    result = False

    if len(email) > 7:
        if re.match("^[_.0-9a-z-]+@([0-9a-z][0-9a-z-]+.)+[a-z]{2,4}$", email) != None:
            result = True

    return result

#--------------------------------------------------------------------------------------------------

def check_username_password(request):

    if request.method == "POST":
        username = request.POST.get("username", "")
        password = request.POST.get("password", "")

        user = auth.authenticate(username = username, password = password)

        if user and user.is_active:
            user.backend = "project.accounts.backends.CustomUserModelBackend"
            auth.login(request, user)
            return HttpResponse("1")

    return HttpResponse("0")

#--------------------------------------------------------------------------------------------------

def username_exists(request, username):
    return HttpResponse("%d" % int(_username_exists(username)))

#--------------------------------------------------------------------------------------------------

def valid_email(request, email = ""):
    return HttpResponse("%d" % int(_valid_email(email)))

#--------------------------------------------------------------------------------------------------

def login(request):

    err = False

    if request.method == "POST":
        next = request.POST.get("next", "/")
        username = request.POST.get("username", "")
        password = request.POST.get("password", "")

        user = auth.authenticate(username = username, password = password)

        if user and user.is_active:
            # correct password and user is marked as active
            user.backend = "project.accounts.backends.CustomUserModelBackend"
            auth.login(request, user)
            return HttpResponseRedirect(next)
        else:
            err = True
    else:
        next = request.GET.get("next", "")

    return render_to_response("user/login.html",
        {
            "next": next,
            "err": err,
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def signout(request):

    auth.logout(request)
    return HttpResponseRedirect('/')

#--------------------------------------------------------------------------------------------------

@login_required
def list(request):

    if not request.user.permissions.can_manage_users:
        return HttpResponseRedirect("/")

    if request.user.permissions.is_server_admin:
        items = CustomUser.objects.all().order_by("creator__username", "username")
    else:
        items = request.user.get_descendants()

    return render_to_response("user/list.html",
        {
            "items": items,
            "active_menu": "users",
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def edit(request, id = 0, profile_edit = False):

    id = int(id)
    new_item = id == 0

    if not new_item:
        try:
            item = CustomUser.objects.get(id = id)
        except:
            new_item = True

    if new_item:
        item = CustomUser()

    editing_self_profile = item == request.user

    if not (request.user.permissions.can_manage_users or editing_self_profile):
        return HttpResponseRedirect("/")

    if request.method == "POST":

        post = lambda key : request.POST.get(key, "")

        if post("operation") == "delete":
            item.delete()
        else:
            from project.main.models import Page

            if new_item:
                perm = UserPermissions()
                perm.save()
                item.permissions = perm
                subs = UserSubscriptions()
                subs.save()
                item.subscriptions = subs
                item.username = post("username")
                item.creator = request.user

            item.name = post("name")
            item.email = post("email")

            if request.POST.has_key("creator"):
                item.creator_id = post("creator")

            pass1 = post("password1")
            pass2 = post("password2")

            if len(pass1) != 0 and pass1 == pass2:
                item.set_password(pass1)

            if not editing_self_profile:
                if request.user.permissions.is_server_admin:
                    item.permissions.is_server_admin = post("is_server_admin") != ""
                if request.user.permissions.can_manage_site_content:
                    item.permissions.can_manage_site_content = post("can_manage_site_content") != ""
                if request.user.permissions.can_manage_plants:
                    item.permissions.can_manage_plants =  post("can_manage_plants") != ""
                if request.user.permissions.can_manage_users:
                    item.permissions.can_manage_users =  post("can_manage_users") != ""
                if request.user.permissions.can_manage_templates:
                    item.permissions.can_manage_templates =  post("can_manage_templates") != ""
                if request.user.permissions.can_manage_controllers:
                    item.permissions.can_manage_controllers =  post("can_manage_controllers") != ""
                if request.user.permissions.can_manage_media:
                    item.permissions.can_manage_media =  post("can_manage_media") != ""
                if request.user.permissions.rw_tags_access:
                    item.permissions.rw_tags_access =  post("rw_tags_access") != ""
                item.is_staff = item.permissions.is_server_admin
                item.is_superuser = item.is_staff
                item.permissions.save()
                
            item.subscriptions.alarms_events_subscription = post("alarms_events_subscription") != ""
            item.subscriptions.save()
                
            item.save()

            if not editing_self_profile or request.user.permissions.can_manage_users:
                homepages = request.POST.get("homepages").split(",")

                # delete deselected homepages
                for p in item.homepages.all():
                    if not "%d" % p.id in homepages:
                        item.homepages.remove(p)
                        # remove user from plant
                        plant = p.get_plant()
                        if plant != None:
                            plant.users.remove(item)

                # add selected pages
                for id in homepages:
                    try:
                        p = Page.objects.get(id = id)
                        if not p in item.homepages.all():
                            item.homepages.add(p)

                        plant = p.get_plant()
                        if plant != None and not item in plant.users.all():
                            plant.users.add(item)
                    except:
                        pass

        return HttpResponseRedirect("/" if profile_edit else "/user/list/")

    plant_pages = request.user.get_all_plants_pages()

    if not new_item:
        homepages = item.homepages.all()
        for p in plant_pages:
            p.is_homepage = p in homepages
    else:
        homepages = None

    from project.main.models import Controller

    return render_to_response("user/edit.html",
        {
            "item": item,
            "plant_pages": plant_pages,
            "user_management": not profile_edit,
            "relay_contents": get_relay_data_str(request, item.id),
            "active_menu": "users",
            "homepages": homepages,
            "controllers": Controller.objects.filter(owner = item).order_by("date_added") if not new_item else [],
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def edit_profile(request):

    return edit(request, request.user.id, True)

#--------------------------------------------------------------------------------------------------

@login_required
def delete(request, id):

    if not request.user.permissions.can_manage_users:
        return HttpResponseRedirect("/")

    try:
        user = CustomUser.objects.get(id = id)
        users = [user] + user.get_descendants()

        for user in users:
            user.delete()
    except:
        raise

    return HttpResponseRedirect("/user/list/")

#--------------------------------------------------------------------------------------------------

@login_required
def toggle_active(request, id):

    if not request.user.permissions.can_manage_users:
        return HttpResponseRedirect("/")

    try:
        user = CustomUser.objects.get(id = id)
        user.is_active = not user.is_active
        user.save()
    except:
        pass

    return HttpResponseRedirect("/user/list/")

#--------------------------------------------------------------------------------------------------

@login_required
def get_available_user_list(request):

    items = CustomUser.objects.filter(is_active = True)

    return render_to_response("user/available_user_list.html",
        {
            "items": items,
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def get_user_list_row(request, id):

    try:
        return render_to_response("user/user_list_row.html",
            {
                "user": CustomUser.objects.get(id = id),
            }, context_instance = RequestContext(request)
        )
    except:
        return HttpResponse("")

#--------------------------------------------------------------------------------------------------

@login_required
def get_relay_data_str(request, id):

    from project.main.models import Relay

    try:
        user = CustomUser.objects.get(id = id)
    except:
        user = None

    try:
        relay = Relay.objects.filter(user = user)
        if relay.count() != 0:
            relay = relay[0]
        else:
            raise
    except:
        relay = None

    return render_to_string("user/relay_contents.html",
        {
            "user": user,
            "relay": relay,
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def get_relay_data(request, id):
    return HttpResponse(get_relay_data_str(request, id))

#--------------------------------------------------------------------------------------------------

@login_required
def set_relay_data(request, id, enabled, session_id):

    from project.main.models import Relay

    try:
        user = CustomUser.objects.get(id = id)

        try:
            relay = Relay.objects.filter(user = user)
            if relay.count() != 0:
                relay = relay[0]
            else:
                raise
        except:
            relay = Relay()
            relay.user = user

        relay.enabled = enabled == "1"
        if session_id != "-1":
            try:
                relay.session_id = int(session_id)
            except:
                pass
        relay.save()
    except:
        pass

    return HttpResponse(get_relay_data_str(request, id))

#--------------------------------------------------------------------------------------------------

@login_required
def reset_relay_data(request, id):

    from project.main.models import Relay

    try:
        user = CustomUser.objects.get(id = id)
        Relay.objects.filter(user = user).update(message_count_tx = 0, message_count_rx = 0, last_controller_nad = 0, last_message = None)
    except:
        pass

    return HttpResponse(get_relay_data_str(request, id))

#--------------------------------------------------------------------------------------------------
