#!/usr/bin/python
#
from django.core import management
import settings as settings
management.setup_environ(settings)

from main.models import Report
import datetime
import re
from smtplib import SMTPException
from email.MIMEImage import MIMEImage
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
import pytz

if __name__ == "__main__":
    reports = Report.objects.all()
    reports = reports.filter(timestamp_sent=None) | reports.filter(timestamp_sent__day=0)
    
    notifications = {'null_user': []}
    for report in reports:
        if report.plant.users.count():
            was_added = False
            for user in report.plant.users.all():
                if (report.type == 1 and user.subscriptions.alarms_events_subscription) and user.email:
                    emails = re.split("[,;: ]", user.email)
                    for email in emails:
                        email = email.strip()
                        if (len(email) > 0):
                            if (not email in notifications.keys()):
                                notifications[email] = []
                            if notifications[email].count(report) == 0:
                                notifications[email].append(report)
                            was_added = True
            if not was_added:
                notifications['null_user'].append(report)
        else:
            notifications['null_user'].append(report)
        
    def get_id_from_report(x):
        return x.id
    
    reports_processed_succ = []
    
    for address in notifications.keys():
        try:
            if not (address == 'null_user'):
                text_content = render_to_string("report/report_mail.txt", {"reports":notifications[address]}) 
                html_content = render_to_string("report/report_mail.html", {"reports":notifications[address]})
                msg = EmailMultiAlternatives(settings.REPORT_EMAIL_SUBJECT, text_content, settings.REPORT_FROM_EMAIL, [address])
                msg.mixed_subtype = 'related'
                msg.attach_alternative(html_content, "text/html")
                try:
                    alarm_image_file = open(settings.MEDIA_ROOT+'../static/img/ico_exclamation_red.png', 'rb')
                    alarm_msg_image = MIMEImage(alarm_image_file.read())
                    alarm_image_file.close()
                    alarm_msg_image.add_header('Content-ID', '<ico_exclamation_red>')
                    msg.attach(alarm_msg_image)
                    event_image_file = open(settings.MEDIA_ROOT+'../static/img/ico_exclamation.png', 'rb')
                    event_msg_image = MIMEImage(event_image_file.read())
                    event_image_file.close()
                    event_msg_image.add_header('Content-ID', '<ico_exclamation>')
                    msg.attach(event_msg_image)
                except:
                    pass
                msg.send(fail_silently=False)
            reports_processed_succ = reports_processed_succ + map(get_id_from_report, notifications[address])
        except SMTPException:
            pass
        
    now = datetime.datetime.now(pytz.utc)
    now = now.replace(tzinfo=None)
    
    reports_processed = Report.objects.filter(id__in=reports_processed_succ)
    try:
        reports_processed.update(timestamp_sent=now)
    except Warning:
        pass